﻿using System.Collections;
using System.Collections.Generic;
using System.Data;
using Vevo.Base.Data;
using Vevo.Base.Domain;
using Vevo.DataAccessLib;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.Blogs;
using Vevo.Domain.DataInterfaces;
using Vevo.Shared.DataAccess;
using Vevo.Shared.Utilities;

namespace Vevo.Data
{
    public class BlogCategoryRepositoryNoCache : IBlogCategoryRepository
    {
        private const string AllBlogCategoryFields =
           "BlogCategory.BlogCategoryID, UrlName, SortOrder, IsEnabled ";

        private const string AllBlogCategoryLocaleFields =
            "BlogCategoryID, CultureID, Name, Description, MetaKeyword, MetaDescription ";

        private const string AllBlogCategoryFieldsWithLocale =
           "BlogCategory.BlogCategoryID, Name, Description, UrlName, IsEnabled, SortOrder, " +
           "MetaKeyword, MetaDescription ";

        private static UrlNameAccessHelper _urlNameHelper = new UrlNameAccessHelper( "BlogCategory", "BlogCategoryID" );

        private BlogCategory Create( BlogCategory blogcategory )
        {
            int recCount = int.Parse( DataAccess.ExecuteScalar(
                "SELECT COUNT(*) FROM BlogCategory;" ) );

            string newID = DataAccess.ExecuteScalar(
                "INSERT INTO BlogCategory ( SortOrder, IsEnabled ) " +
                "VALUES ( @SortOrder, @IsEnabled ); " +
                "SELECT @@Identity; ",
                DataAccess.CreateParameterInt32( recCount++ ),
                DataAccess.CreateParameterBool( blogcategory.IsEnabled )
                );

            blogcategory.BlogCategoryID = newID;

            blogcategory.UrlName = _urlNameHelper.UpdateUrlName(
                blogcategory.BlogCategoryID.ToString(),
                blogcategory.Name );

            InsertBlogCategoryLocales( blogcategory );

            return blogcategory;
        }

        private void InsertBlogCategoryLocales( BlogCategory blogcategory )
        {
            foreach (BlogCategoryLocale locale in blogcategory.Locales)
            {
                DataAccess.ExecuteNonQuery(
                    "INSERT INTO BlogCategoryLocale ( BlogCategoryID, CultureID, Name, Description, MetaKeyword, MetaDescription ) " +
                    "VALUES ( @BlogCategoryID, @CultureID, @Name, @Description, @MetaKeyword, @MetaDescription ); " +
                    "SELECT @@Identity; ",
                    DataAccess.CreateParameterString( blogcategory.BlogCategoryID ),
                    DataAccess.CreateParameterString( locale.CultureID ),
                    DataAccess.CreateParameterString( locale.Name ),
                    DataAccess.CreateParameterString( locale.Description ),
                    DataAccess.CreateParameterString( locale.MetaKeyword ),
                    DataAccess.CreateParameterString( locale.MetaDescription ) );
            }
        }

        private BlogCategory Update( BlogCategory blogcategory )
        {
            int mainUpdate = DataAccess.ExecuteNonQuery(
                "UPDATE BlogCategory " +
                "SET IsEnabled = @IsEnabled WHERE BlogCategoryID = @BlogCategoryID; ",
                DataAccess.CreateParameterBool( blogcategory.IsEnabled ),
                DataAccess.CreateParameterString( blogcategory.BlogCategoryID ) );

            if (mainUpdate == 0)
                throw new DataAccessException(
                    "BlogCategory Update Failed. Not Found BlogCategory with ID: " + blogcategory.BlogCategoryID +
                    ", Name: " + blogcategory.Name );

            blogcategory.Locales.Load();
            DeleteLocales( blogcategory.BlogCategoryID );

            blogcategory.UrlName = _urlNameHelper.UpdateUrlName(
                blogcategory.BlogCategoryID.ToString(),
                blogcategory.Name );
            InsertBlogCategoryLocales( blogcategory );

            return blogcategory;
        }

        private int GetAllCount(
            Culture culture,
            string sortBy,
            BoolFilter isEnabled )
        {
            string query = "SELECT ROW_NUMBER() over (" + CreateSortByText( sortBy ) + ") as rownum, * FROM ( " +
                "SELECT " + AllBlogCategoryFieldsWithLocale +
                "FROM BlogCategory INNER JOIN " +
                "     (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L " +
                "ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "BlogCategory.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ") AS BlogCategory";

            string getCountManu = "with SearchResult as (" + query + ") " +
                " select COUNT(rownum) from SearchResult;";

            string countResult = DataAccess.ExecuteScalar( getCountManu,
                DataAccess.CreateParameterString( culture.CultureID ) );

            return ConvertUtilities.ToInt32( countResult );
        }

        private string CreateSortByText( string sortBy )
        {
            string field = "blogcategoryid";
            if (sortBy == null || sortBy.Trim() == "")
                return "";
            else if (sortBy.Trim().ToLower().StartsWith( field ))
                return "ORDER BY BlogCategory." + sortBy.Trim() + " ";
            else
                return "ORDER BY " + sortBy + " ";
        }

        private string ConvertFieldName( string fieldName )
        {
            string field = "blogcategoryid";
            if (fieldName.ToLower() == field)
            {
                return "BlogCategory.BlogCategoryID";
            }
            else
            {
                return fieldName;
            }
        }

        
        private IList<BlogCategory> ConvertBlogCategoryTableToList( Culture defaultCulture, DataTable table )
        {
            IList<BlogCategory> list = new List<BlogCategory>();

            foreach (DataRow row in table.Rows)
                list.Add( CreateOneBlogCategoryFromRow( defaultCulture, row ) );

            return list;
        }

        private List<ILocale> ConvertBlogCategoryLocaleTableToList( DataTable localeTable )
        {
            List<ILocale> list = new List<ILocale>();

            foreach (DataRow row in localeTable.Rows)
                list.Add( CreateOneBlogCategoryLocaleFromRow( row ) );

            return list;
        }

        private BlogCategory CreateOneBlogCategoryFromRow( Culture defaultCulture, DataRow row )
        {
            BlogCategory blogcategory = new BlogCategory( defaultCulture );

            // Data from BlogCategory database table
            blogcategory.BlogCategoryID = row["BlogCategoryID"].ToString();
            blogcategory.UrlName = row["UrlName"].ToString();
            blogcategory.IsEnabled = (bool) row["IsEnabled"];
            blogcategory.SortOrder = (int) row["SortOrder"];

            // Data from BlogCategoryLocale database table
            blogcategory.Name = row["Name"].ToString();
            blogcategory.Description = row["Description"].ToString();
            blogcategory.MetaKeyword = row["MetaKeyword"].ToString();
            blogcategory.MetaDescription = row["MetaDescription"].ToString();

            return blogcategory;
        }

        private BlogCategoryLocale CreateOneBlogCategoryLocaleFromRow( DataRow row )
        {
            BlogCategoryLocale locale = new BlogCategoryLocale();
            locale.CultureID = row["CultureID"].ToString();
            locale.Name = row["Name"].ToString();
            locale.Description = row["Description"].ToString();
            locale.MetaKeyword = row["MetaKeyword"].ToString();
            locale.MetaDescription = row["MetaDescription"].ToString();

            return locale;
        }

        #region Public Methods

        public void DeleteLocales( string blogcategoryID )
        {
            DataAccess.ExecuteNonQuery(
                "DELETE FROM BlogCategoryLocale " +
                "WHERE BlogCategoryID = @BlogCategoryID; ",
                DataAccess.CreateParameterString( blogcategoryID ) );
        }

        public BlogCategory Save( BlogCategory blogcategory )
        {
            BlogCategory result;
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                if (blogcategory.BlogCategoryID == "0")
                    result = Create( blogcategory );
                else
                    result = Update( blogcategory );

                transaction.Complete();
            }

            return result;
        }

        public void Delete( string blogcategoryID )
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery(
                    "DELETE FROM BlogCategory WHERE BlogCategoryID = @BlogCategoryID; ",
                    DataAccess.CreateParameterString( blogcategoryID )
                    );
                DeleteLocales( blogcategoryID );
                transaction.Complete();
            }
        }

        public IList<ILocale> GetLocales( string blogcategoryID )
        {
            DataTable localeTable = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCategoryLocaleFields +
                "FROM BlogCategoryLocale WHERE BlogCategoryID = @BlogCategoryID; ",
                DataAccess.CreateParameterString( blogcategoryID ) );

            return ConvertBlogCategoryLocaleTableToList( localeTable );
        }

        public void DeleteAll()
        {
            using (DataTransactionScope transaction = new DataTransactionScope())
            {
                DataAccess.ExecuteNonQuery( "DELETE FROM BlogCategory;" );
                DataAccess.ExecuteNonQuery( "DELETE FROM BlogCategoryLocale;" );
                transaction.Complete();
            }
        }

        public BlogCategory GetOne( Culture culture, string blogcategoryID )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCategoryFieldsWithLocale + "FROM BlogCategory " +
                " LEFT JOIN (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L  " +
                " ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                "WHERE BlogCategory.BlogCategoryID = @ID; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( blogcategoryID ) );

            if (table.Rows.Count > 0)
                return CreateOneBlogCategoryFromRow( culture, table.Rows[0] );
            else
                return NullBlogCategory.Instance;
        }

        public string GetBlogCategoryIDFromBlogCategoryName( string blogcategoryName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT BlogCategoryID FROM BlogCategoryLocale " +
                "WHERE Name = @Name ;",
                DataAccess.CreateParameterString( blogcategoryName ) );

            if (table.Rows.Count > 0)
                return table.Rows[0]["BlogCategoryID"].ToString();
            else
                return string.Empty;
        }

        public string GetBlogCategoryIDFromBlogCategoryURL(string blogCategoryURL)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT BlogCategoryID FROM BlogCategory " +
                "WHERE UrlName = @Name ;",
                DataAccess.CreateParameterString(blogCategoryURL));

            if (table.Rows.Count > 0)
                return table.Rows[0]["BlogCategoryID"].ToString();
            else
                return string.Empty;
        }

        public string GetBlogCategoryNameFromBlogCategoryURL(string blogCategoryURL)
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT L.Name FROM BlogCategory B, BlogCategoryLocale L WHERE L.BlogCategoryID = B.BlogCategoryID "+
                    "AND UrlName = @UrlName ;",
                DataAccess.CreateParameterString(blogCategoryURL));

            if (table.Rows.Count > 0)
                return table.Rows[0]["Name"].ToString();
            else
                return string.Empty;
        }

        public IList<BlogCategory> GetAll( Culture culture, BoolFilter isEnabled, string sortBy )
        {
            DataTable table = DataAccess.ExecuteSelect(
                 "SELECT " + AllBlogCategoryFieldsWithLocale +
                 "FROM BlogCategory LEFT JOIN " +
                 "     (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L " +
                 "ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                 FlagFilterHelper.CreateFlagFilterWithWhere( "BlogCategory.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                 CreateSortByText( sortBy ) + "; ",
                 DataAccess.CreateParameterString( culture.CultureID )
                 );

            return ConvertBlogCategoryTableToList( culture, table );
        }

        public IList<BlogCategory> GetAll(
             Culture culture,
             string sortBy,
             BoolFilter isEnabled,
             int startIndex,
             int endIndex,
             out int howManyItems )
        {
            string query = "SELECT ROW_NUMBER() over (" + CreateSortByText( sortBy ) + ") as rownum, * FROM ( " +
                "SELECT " + AllBlogCategoryFieldsWithLocale +
                "FROM BlogCategory INNER JOIN " +
                "     (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L " +
                "ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                FlagFilterHelper.CreateFlagFilterWithAnd( "BlogCategory.IsEnabled", RepositoryUtilities.ConvertToFlagFilter( isEnabled ) ) +
                ") AS BlogCategory";

            string getManuItemsQuery = "with SearchResult as (" + query + ") " +
                " select * from SearchResult where rownum between " + (startIndex + 1).ToString() + " and " + (endIndex + 1).ToString() + ";";

            DataTable full = DataAccess.ExecuteSelect( getManuItemsQuery,
                DataAccess.CreateParameterString( culture.CultureID ) );
            howManyItems = GetAllCount( culture, sortBy, isEnabled );

            return ConvertBlogCategoryTableToList( culture, full );
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            DataTable table = DataAccess.ExecuteSelect(
                    "SELECT BlogCategory.BlogCategoryID, Name,Description, UrlName, IsEnabled, " +
                    " MetaKeyword, MetaDescription " +
                    "FROM BlogCategory LEFT JOIN BlogCategoryLocale L " +
                    "    ON BlogCategory.BlogCategoryID = L.BlogCategoryID WHERE 0 = 1; " );

            DataColumn[] keys = new DataColumn[] { table.Columns["BlogCategoryID"] };
            table.PrimaryKey = keys;
            return RepositoryUtilities.ConvertTableToSchemaItemList( table );
        }

        public IList<BlogCategory> SearchBlogCategory(
            Culture culture,
            string sortBy,
            SearchFilter searchFilter,
            BoolFilter isActive,
            out int howManyItems )
        {
            string fieldName = ConvertFieldName( searchFilter.FieldName );
            ArrayList parameterList = new ArrayList();
            SearchFilterHelperNew searchFilterHelper = new SearchFilterHelperNew();
            parameterList.Add( DataAccess.CreateParameterString( culture.CultureID ) );
            string commandSearch = searchFilterHelper.CreateCommandWithWhere(
                searchFilter, fieldName, parameterList );

            string query = "SELECT " + AllBlogCategoryFieldsWithLocale +
                "FROM BlogCategory LEFT JOIN " +
                "     (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L " +
                "     ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                commandSearch +
                CreateSortByText( sortBy ) + "; ";

            DataTable full = DataAccess.ExecuteSelect( query,
                DataAccess.ConvertArrayListToDbParameters( parameterList )
                );

            DataColumn[] keys = new DataColumn[] { full.Columns["BlogCategoryID"] };
            full.PrimaryKey = keys;

            howManyItems = full.Rows.Count;
            return ConvertBlogCategoryTableToList( culture, full );
        }

        public void UpdateSortOrder( string[] sortedBlogCategoryIDs )
        {
            int sortCount = 0;
            foreach (string items in sortedBlogCategoryIDs)
            {
                sortCount++;
                DataAccess.ExecuteNonQuery(
                    "UPDATE BlogCategory SET SortOrder = @SortOrder WHERE BlogCategoryID = @BlogCategoryID;",
                    DataAccess.CreateParameterString( sortCount.ToString() ),
                    DataAccess.CreateParameterString( items ) );
            }
        }

        public BlogCategory GetOneByUrlName( Culture culture, string urlName )
        {
            DataTable table = DataAccess.ExecuteSelect(
                "SELECT " + AllBlogCategoryFieldsWithLocale +
                " FROM BlogCategory LEFT JOIN  " +
                "     (SELECT * FROM BlogCategoryLocale WHERE CultureID = @CultureID) AS L  " +
                "       ON BlogCategory.BlogCategoryID = L.BlogCategoryID " +
                "WHERE UrlName = @UrlName; ",
                DataAccess.CreateParameterString( culture.CultureID ),
                DataAccess.CreateParameterString( urlName ) );

            if (table.Rows.Count > 0)
                return CreateOneBlogCategoryFromRow( culture, table.Rows[0] );
            else
                return NullBlogCategory.Instance;
        }
        #endregion
    }
}
