using System;
using System.Collections.Generic;
using System.Text;
using System.Web.UI;
using Vevo.Base.Data.Caching;
using Vevo.Domain;
using Vevo.Domain.Base;
using Vevo.Domain.DataInterfaces;
using Vevo.Domain.Products;
using Vevo.Shared.Utilities;
using Vevo.Base.Domain;

namespace Vevo.Data
{
    public class OptionItemRepository : IOptionItemRepository
    {
        #region Private

        private const string RepositoryName = "OptionItemRepository";

        private const string GetOneSignature = "GetOne";
        private const string GetLocalesSignature = "GetLocales";
        private const string GetByOptionGroupIDSignature = "GetByOptionGroupIDSignature";
        private const string SearchOptionItemSignature = "SearchOptionItem";
        private const string GetTableSchemaSignature = "GetTableSchema";

        private DataCache _dataCache = new DataCache( RepositoryName );

        private OptionItemRepositoryNoCache _repository = new OptionItemRepositoryNoCache();
        private ICacheDependencyUtility _cacheDependency
            = new CacheDependencyUtility( new string[] { "OptionItemLocale", "OptionItem" } );

        #endregion

        #region Constructors

        public OptionItemRepository()
        {
        }

        public OptionItemRepository( ICacheDependencyUtility cacheDependency )
        {
            _cacheDependency = cacheDependency;
        }

        #endregion

        #region Public Methods

        public OptionItem Save( OptionItem optionItem )
        {
            // Need to write to database first before clearing cache.
            // Otherwise, cache may remember incorrect values before saving.
            OptionItem newOptionItem = _repository.Save( optionItem );
            _dataCache.ClearRepositoryCache();
            return newOptionItem;
        }

        public void Delete( string optionItemID )
        {
            _repository.Delete( optionItemID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteByOptionGroupID( string optionGroupID )
        {
            _repository.DeleteByOptionGroupID( optionGroupID );
            _dataCache.ClearRepositoryCache();
        }

        public void DeleteLocalesByCultureID( string cultureID )
        {
            _repository.DeleteLocalesByCultureID( cultureID );
            _dataCache.ClearRepositoryCache();
        }

        public IList<ILocale> GetLocales( string optionItemID )
        {
            IList<ILocale> list = (IList<ILocale>) _dataCache.GetValue( GetLocalesSignature, _cacheDependency, optionItemID );

            if (list == null)
            {
                list = _repository.GetLocales( optionItemID );
                _dataCache.SetValue( GetLocalesSignature, list, optionItemID );
            }

            return list;
        }

        public OptionItem GetOne( Culture culture, string optionItemID )
        {
            OptionItem result = (OptionItem) _dataCache.GetValue( GetOneSignature, _cacheDependency, culture.CultureID, optionItemID );

            if (result == null)
            {
                result = _repository.GetOne( culture, optionItemID );
                _dataCache.SetValue( GetOneSignature, result, culture.CultureID, optionItemID );
            }

            return result;
        }

        public IList<OptionItem> GetByOptionGroupID( Culture culture, string optionGroupID )
        {
            IList<OptionItem> list = (IList<OptionItem>) _dataCache.GetValue(
                GetByOptionGroupIDSignature, _cacheDependency, culture.CultureID, optionGroupID );

            if (list == null)
            {
                list = _repository.GetByOptionGroupID( culture, optionGroupID );
                _dataCache.SetValue( GetByOptionGroupIDSignature, list, culture, optionGroupID );
            }

            return list;
        }

        public IList<OptionItem> SearchOptionItem(
                Culture culture,
                string optionGroupID,
                string sortBy,
                SearchFilter searchFilter,
                int startIndex,
                int endIndex,
                out int howManyItems
            )
        {
            howManyItems = 0;
            Pair result = (Pair) _dataCache.GetValue(
                SearchOptionItemSignature, _cacheDependency,
                culture.CultureID,
                optionGroupID,
                sortBy,
                searchFilter.ToString(),
                startIndex,
                endIndex
                );

            if (result == null)
            {
                result = new Pair();
                result.First = _repository.SearchOptionItem(
                    culture, optionGroupID, sortBy, searchFilter, startIndex, endIndex, out howManyItems );
                result.Second = howManyItems;
                _dataCache.SetValue(
                    SearchOptionItemSignature,
                    result,
                    culture.CultureID,
                    optionGroupID,
                    sortBy,
                    searchFilter.ToString(),
                    startIndex,
                    endIndex );
            }

            howManyItems = ConvertUtilities.ToInt32( result.Second );
            return (IList<OptionItem>) result.First;
        }

        public IList<TableSchemaItem> GetTableSchema()
        {
            IList<TableSchemaItem> list = (IList<TableSchemaItem>) _dataCache.GetValue(
                   GetTableSchemaSignature, _cacheDependency
                   );

            if (list == null)
            {
                list = _repository.GetTableSchema();
                _dataCache.SetValue(
                    GetTableSchemaSignature,
                    list
                    );
            }

            return list;
        }

        #endregion
    }
}
