using System;
using System.Data;
using System.Collections.Generic;
using System.Configuration;
using System.Text.RegularExpressions;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Xml;
using GCheckout.Checkout;
using GCheckout.Util;
using Vevo.Domain;
using Vevo.Domain.Configurations;
using Vevo.Domain.Orders;
using Vevo.Domain.Shipping;
using Vevo.Domain.Shipping.Custom;
using Vevo.Domain.Tax;
using Vevo.Domain.Payments.SessionState;
using Vevo.Domain.Users;
using Vevo.Shared.SystemServices;
using Vevo.WebAppLib;
using Vevo.Domain.Stores;
using Vevo.Base.Domain;
using Vevo.Deluxe.Domain.Orders;

namespace Vevo.Domain.Payments.Google
{
    /// <summary>
    /// Summary description for GoogleCheckoutRequestHelper
    /// </summary>
    public class GoogleCheckoutRequestHelper
    {
        #region Private
        private const string PrefixShippingName = "vevo";


        private XmlDocument _tempXmlDocument;


        private XmlDocument TempXmlDocument
        {
            get
            {
                if (_tempXmlDocument == null)
                    _tempXmlDocument = new XmlDocument();

                return _tempXmlDocument;
            }
        }


        private XmlNode CreateMerchantPrivateDataNode( string tagName, string val )
        {
            XmlNode tag = TempXmlDocument.CreateElement( tagName );
            tag.InnerText = val;
            return tag;
        }

        private string CreateCallbackUrl( string webAppUrl )
        {
            string urlCallBack = webAppUrl + "Gateway/GoogleCallback.aspx";
            if (WebConfiguration.GoogleEnvironment.ToLower() != "sandbox" &&
                urlCallBack.ToLower().StartsWith( "http://" ))
            {
                Regex regex = new Regex( "http://", RegexOptions.IgnoreCase );
                urlCallBack = regex.Replace( urlCallBack, "https://" );
            }

            return urlCallBack;
        }

        private void AddMerchantCalculatedTax(
            CheckoutShoppingCartRequest request,
            Cart cart,
            WholesaleStatus wholesaleStatus )
        {
            decimal totalLineItemTaxes = 0;

            foreach (ICartItem item in cart.GetCartItems())
            {
                totalLineItemTaxes += item.GetCheckoutIncludedTaxAmount( wholesaleStatus );
            }

            request.AddMerchantPrivateDataNode(
                CreateMerchantPrivateDataNode( "totalLineItemTaxes", totalLineItemTaxes.ToString( "f2" ) ) );

            decimal subtotal = cart.GetSubtotal( wholesaleStatus );

            request.AddMerchantPrivateDataNode(
                CreateMerchantPrivateDataNode( "subtotal", Currency.RoundAmount( subtotal ).ToString() ) );
        }

        private void AddDefaultTaxRule( CheckoutShoppingCartRequest request )
        {
            request.MerchantCalculatedTax = true;

            double currentTax = DataAccessContext.Configurations.GetDoubleValue( "TaxPercentage" ) / 100.0;
            switch (DataAccessContext.Configurations.GetValue( "TaxRule" ))
            {
                case "2":
                    request.AddCountryTaxRule(
                        GCheckout.AutoGen.USAreas.ALL,
                        currentTax,
                        DataAccessContext.Configurations.GetBoolValue( "IsTaxableShippingCost" ) );
                    break;

                case "3":
                    request.AddStateTaxRule(
                        DataAccessContext.Configurations.GetValue( "TaxLocation" ),
                        currentTax,
                        DataAccessContext.Configurations.GetBoolValue( "IsTaxableShippingCost" ) );
                    break;

                default:
                    request.AddCountryTaxRule(
                        GCheckout.AutoGen.USAreas.ALL,
                        currentTax,
                        DataAccessContext.Configurations.GetBoolValue( "IsTaxableShippingCost" ) );
                    break;
            }
        }

        private void AddCartItems(
            CheckoutShoppingCartRequest request,
            Cart cart,
            Culture culture,
            Currency currency,
            WholesaleStatus wholesaleStatus )
        {
            foreach (ICartItem item in cart.GetCartItems())
            {
                if (item.Options == null)
                    request.AddItem(
                        item.GetName( culture, currency )
                            .Replace( "<br>", "" ),
                        String.Empty,
                        item.ProductID,
                        item.GetCheckoutUnitPrice( wholesaleStatus ),
                        item.Quantity,
                        CreateMerchantPrivateDataNode( "NameOriginal",
                            item.GetName( culture, currency ) ),
                        CreateMerchantPrivateDataNode( "OptionItemID", String.Empty ) );
                else
                {
                    request.AddItem(
                        item.GetName( culture, currency )
                            .Replace( "<br>", "" ),
                        String.Empty,
                        item.ProductID,
                        item.GetCheckoutUnitPrice( wholesaleStatus ),
                        item.Quantity,
                        CreateMerchantPrivateDataNode(
                            "NameOriginal",
                            item.GetName( culture, currency ) ),
                        CreateMerchantPrivateDataNode( "OptionItemID", item.Options.ToString() ) );
                }
            }
        }

        private ShippingRestrictions GetShippingRestriction( ShippingOption shippingOption )
        {
            ShippingRestrictions shippingRestrictions = new ShippingRestrictions();

            foreach (ShippingZone shippingZone in shippingOption.ShippingZone)
            {
                IList<ShippingZoneItem> shippingZoneItemList =
                    DataAccessContext.ShippingZoneGroupRepository.GetShippingZoneItemByZoneGroupID( shippingZone.ZoneGroupID );

                foreach (ShippingZoneItem shippingZoneItem in shippingZoneItemList)
                {
                    string countryCode = shippingZoneItem.CountryCode;
                    string stateCode = shippingZoneItem.StateCode;
                    string zipCode = shippingZoneItem.ZipCode;

                    switch (shippingOption.AllowedType)
                    {
                        case ShippingOption.ShippingZoneAllowedType.Allow:
                            {
                                if (!string.IsNullOrEmpty( zipCode ))
                                    shippingRestrictions.AddAllowedPostalArea( countryCode, zipCode );
                                else
                                {
                                    if (countryCode.ToLower() != "us")
                                        shippingRestrictions.AddAllowedPostalArea( countryCode );
                                    else
                                    {
                                        if (!string.IsNullOrEmpty( stateCode ))
                                            shippingRestrictions.AddAllowedStateCode( stateCode );
                                        else
                                            shippingRestrictions.AddAllowedPostalArea( countryCode );
                                    }
                                }
                                break;
                            }
                        case ShippingOption.ShippingZoneAllowedType.Deny:
                            {
                                if (!string.IsNullOrEmpty( zipCode ))
                                    shippingRestrictions.AddExcludedPostalArea( countryCode, zipCode );
                                else
                                {
                                    if (countryCode.ToLower() != "us")
                                        shippingRestrictions.AddExcludedPostalArea( countryCode );
                                    else
                                    {
                                        if (!string.IsNullOrEmpty( stateCode ))
                                            shippingRestrictions.AddExcludedStateCode( stateCode );
                                        else
                                            shippingRestrictions.AddExcludedPostalArea( countryCode );
                                    }
                                }
                                break;
                            }
                        default:
                            {
                                shippingRestrictions.AddAllowedWorldArea();
                                break;
                            }
                    }
                }
            }

            return shippingRestrictions;
        }

        private void AddMerchantCalculatedShipping(
            CheckoutShoppingCartRequest request,
            Cart cart,
            Culture culture,
            WholesaleStatus wholesaleStatus )
        {
            IList<ShippingOption> shippingList = DataAccessContext.ShippingOptionRepository.GetShipping(
                culture, BoolFilter.ShowFalse );
            decimal price;
            decimal shippingCost;
            decimal handlingFee;

            for (int i = 0; i < shippingList.Count; i++)
            {
                string merchantCountry = DataAccessContext.Configurations.GetValue( "ShippingMerchantCountry" );
                Address merchantAddress = new Address( "", "", "", "", "", "", "", "", merchantCountry, "", "" );

                ShippingOption shippingOption = DataAccessContext.ShippingOptionRepository.GetOne(
                    culture,
                    shippingList[i].ShippingID );

                ShippingMethod shippingMethod = shippingOption.CreateNonRealTimeShippingMethod();

                OrderCalculator orderCalculator = new OrderCalculator();

                shippingCost = orderCalculator.GetShippingCost(
                    shippingMethod,
                    cart.SeparateCartItemGroups(),
                    wholesaleStatus,
                    0.0m ) + CartItemPromotion.GetShippingCostFromPromotion( shippingMethod,
                    cart.SeparateCartItemGroups(),
                    wholesaleStatus,
                    0.0m );

                handlingFee = orderCalculator.GetHandlingFee(
                    shippingMethod,
                    cart.SeparateCartItemGroups(),
                    wholesaleStatus );

                price = shippingCost + handlingFee;

                ShippingRestrictions shippingRestriction = GetShippingRestriction( shippingList[i] );

                request.AddMerchantCalculatedShippingMethod( shippingList[i].ShippingName, price, shippingRestriction, shippingRestriction );

                request.AddMerchantPrivateDataNode(
                    CreateMerchantPrivateDataNode(
                        EncodeFormatXml( shippingList[i].ShippingName ), price.ToString( "f2" ) ) );
            }
        }

        private void AddShoppingCartSession( CheckoutShoppingCartRequest request, CheckoutDetails checkoutDetails, Cart cart )
        {
            string ticket = SessionPool.CreateSession();
            Session session = SessionPool.GetSession( ticket );

            session[GoogleCheckoutPaymentMethod.CheckoutDetailsSessionName] = checkoutDetails;
            session[GoogleCheckoutPaymentMethod.CartSessionName] = cart;

            request.AddMerchantPrivateDataNode(
                CreateMerchantPrivateDataNode( PaymentOption.GoogleCheckoutSessionTicketName, ticket ) );
        }
        #endregion


        public static string EncodeFormatXml( string source )
        {
            return XmlConvert.EncodeName( PrefixShippingName + source );
        }

        public static string DecodeFormatXml( string source )
        {
            return XmlConvert.DecodeName( source.Substring( PrefixShippingName.Length ) );
        }

        public void PopulateRequest(
            string webAppUrl,
            CheckoutShoppingCartRequest request,
            Cart cart,
            Culture culture,
            Currency currency,
            WholesaleStatus wholesaleStatus,
            CheckoutDetails checkoutDetails
            )
        {
            request.AcceptMerchantCoupons = true;
            request.AcceptMerchantGiftCertificates = true;

            request.MerchantCalculationsUrl = CreateCallbackUrl( webAppUrl );

            AddCartItems( request, cart, culture, currency, wholesaleStatus );

            AddDefaultTaxRule( request );
            AddMerchantCalculatedTax( request, cart, wholesaleStatus );

            AddMerchantCalculatedShipping( request, cart, culture, wholesaleStatus );

            AddShoppingCartSession( request, checkoutDetails, cart );

            request.ContinueShoppingUrl = webAppUrl;
            request.AddMerchantPrivateDataNode( CreateMerchantPrivateDataNode( "StoreID", new StoreRetriever().GetCurrentStoreID() ) );
            GoogleCheckoutPaymentMethod payment = new GoogleCheckoutPaymentMethod();
        }

    }
}
