﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using LumenWorks.Framework.IO.Csv;
using Vevo.Domain.Products;
using Vevo.Shared.Domain.SystemServices;
using Vevo.Shared.SystemServices;

namespace Vevo.Domain.ImportExport
{
    public class ProductSpecificationImporter
    {
        #region Private

        private Dictionary<string, string> _skuDatabase;

        private const int _skuIndex = 0;
        private const int _specificationItemIDIndex = 1;
        private const int _valueIndex = 2;

        private Culture _culture;
        private IFileManager _fileManager;
        private ProductImportStatus _status;

        private DateTime _timeStart;
        private DateTime _timeEnd;

        private string _storeID;

        private CsvReader CreateCsvReader( string relativePath, out TextReader reader )
        {
            Encoding encoding;
            if (_fileManager.ContainsUnicodeBom( relativePath ))
                encoding = Encoding.Unicode;
            else
                encoding = Encoding.Default;

            reader = _fileManager.CreateTextReader( relativePath, encoding );
            CsvReader csv = new CsvReader( reader, true );
            csv.MissingFieldAction = MissingFieldAction.ReplaceByEmpty;
            csv.ParseError += ParseErrorHandler;

            return csv;
        }

        private void ParseErrorHandler( object sender, ParseErrorEventArgs e )
        {
            _status.AddError();
        }

        private bool IsSpecificationItemIDValid( string specificationItemID, string sku )
        {
            SpecificationItem specificationItem = DataAccessContext.SpecificationItemRepository.GetOne( _culture, specificationItemID );

            if (specificationItem.IsNull)
                return false;
            else
            {
                Product product = DataAccessContext.ProductRepository.GetOneBySku( _culture, _storeID, sku );
                if (product.ProductSpecifications.Count > 0)
                {
                    if (product.GetSpecificationGroupID( _culture ) == specificationItem.SpecificationGroupID)
                        return true;
                    else
                        return false;
                }
                else
                    return true;
            }

        }

        private bool IsProductExist( string sku )
        {
            if (_skuDatabase.ContainsKey( sku ))
                return true;
            else
                return false;
        }

        private ProductImportStatus ImportSpecification(
          string filePath,
          out TimeSpan timeDiff,
          string storeID
          )
        {
            TextReader reader = null;
            CsvReader csv = null;
            _status = new ProductImportStatus();
            try
            {
                csv = CreateCsvReader( filePath, out reader );

                // Read data line-by-line
                int i = 2;
                while (csv.ReadNextRecord())
                {
                    try
                    {
                        if (!IsProductExist( csv[_skuIndex] ))
                        {
                            _status.AddProductSKUError( i.ToString() );
                        }
                        else if (!IsSpecificationItemIDValid( csv[_specificationItemIDIndex], csv[_skuIndex] ))
                        {
                            _status.AddSpecificationItemError( i.ToString() );
                        }
                        else
                        {
                            if (ImportSpecificationValue( csv, storeID ))
                            {
                                _status.AddSuccess();
                            }
                        }
                    }
                    catch (Exception ex)
                    {
                        SaveLogFile.SaveLog( ex );
                        _status.AddOtherError( i.ToString(), ex );
                    }
                    i++;
                }

                _status.BuildSpecificationItemError();
                _status.BuildProductSKUError();
            }
            catch (Exception ex)
            {
                SaveLogFile.SaveLog( ex );
                _status.SetFatalError( ex.Message );
            }
            finally
            {
                if (csv != null)
                    csv.Dispose();

                if (reader != null)
                    reader.Dispose();
            }

            _timeEnd = DateTime.Now;
            timeDiff = _timeEnd.Subtract( _timeStart );
            return _status;
        }

        private bool ImportSpecificationValue( CsvReader csv, string storeID )
        {
            bool result = false;
            string sku = csv[_skuIndex].ToString();
            Product product = DataAccessContext.ProductRepository.GetOneBySku( _culture, storeID, sku );

            result = SaveProductSpecificationValue( product, csv, storeID );

            return result;
        }

        private bool SaveProductSpecificationValue( Product product, CsvReader csv, string storeID )
        {
            string specificationItemID = csv[_specificationItemIDIndex].ToString();
            string specificationValue = csv[_valueIndex].ToString();
            if (!String.IsNullOrEmpty( specificationValue ) && !String.IsNullOrEmpty( specificationItemID ))
            {
                product.SetProductSpecificationItemValue( specificationItemID, specificationValue );
                product = DataAccessContext.ProductRepository.Save( product );
            }
            return true;
        }

        #endregion

        #region Constructors

        public ProductSpecificationImporter( Culture culture, IFileManager fileManager )
        {
            _culture = culture;
            _fileManager = fileManager;
        }

        public ProductSpecificationImporter( Culture culture )
        {
            _culture = culture;
            _fileManager = new FileManager();
        }

        #endregion

        #region Public Methods

        public ProductImportStatus ImportSpecificationPurgeAll(
            string filePath, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            _skuDatabase = DataAccessContext.ProductRepository.GetProductIDAndSku();
            DataAccessContext.ProductRepository.DeleteProductSpecification();
            return ImportSpecification( filePath, out timeDiff, storeID );
        }

        public ProductImportStatus ImportSpecificationOverwrite(
            string filePath, out TimeSpan timeDiff, string storeID )
        {
            _timeStart = DateTime.Now;
            _storeID = storeID;
            _skuDatabase = DataAccessContext.ProductRepository.GetProductIDAndSku();
            return ImportSpecification( filePath, out timeDiff, storeID );
        }

        #endregion
    }
}
